-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...;
local functions = namespace.functions;
local variables = namespace.variables;
local llrConfigWindow = CreateFrame("FRAME", "LPTConfigWindow", UIParent);
local userScale = 1;
local leaderScale = 1;
local prefixStatus = C_ChatInfo.RegisterAddonMessagePrefix("lptlootroll");
local loadFrame = CreateFrame("FRAME");


-----------------------
--FUNCTION DEFINITIONS-
-----------------------


--Check button factory.
local function createCheckbutton(name, parent, x_loc, y_loc, displayname, tooltip)
	local checkbutton = CreateFrame("CheckButton", name, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	_G[checkbutton:GetName().."Text"]:SetText(displayname);
	checkbutton.tooltip = tooltip;

	return checkbutton;
end

--Function for creating the roll setting frames.
local function editBoxFactory(frame, frameEditbox, x, y, title, linkedFrame)
	frame:SetPoint("BOTTOMLEFT", llrConfigWindow, "BOTTOMLEFT", x, y);
	frame:SetSize(100,200);
	
	--Create and set title of the setting.
	frame.title = frame:CreateFontString(nil,"OVERLAY");
	frame.title:SetFontObject("GameFontHighLight");
	frame.title:SetPoint("CENTER", frame, "CENTER", 0, 5);
	frame.title:SetText(title);

	--Create the editbox for the setting.
	frame.editBox = frameEditbox;
	frame.editBox:SetPoint("CENTER", frame, "CENTER", 0, -15);
	frame.editBox:SetSize(40,40);
	frame.editBox:SetAutoFocus(false);
	frame.editBox:SetNumeric(true);
	frame.editBox:SetMaxLetters(4);
	frame.editBox:SetFontObject("GameFontHighLight");
	frame.editBox:SetScript("OnKeyDown", function (self, key)
		if key == "TAB" then
			linkedFrame.editBox:SetFocus();
		end
	end);
end

--Function for creating the sliders.
local function sliderFactory(frame, frameSlider, x, y, title)
	frameSlider.tooltipText = "Sets scale (1 is default).";
	frameSlider:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", x, y);
	frameSlider:SetMinMaxValues(0.5, 2);
	frameSlider:SetWidth(120);
	frameSlider:SetOrientation("HORIZONTAL");
	frameSlider.stepValue = 0.1;
	frameSlider:SetValueStep(frameSlider.stepValue);
	
	_G[frameSlider:GetName().."Low"]:SetText("0.5");
	_G[frameSlider:GetName().."High"]:SetText("2");
	
	frameSlider.title = frameSlider:CreateFontString(nil, "OVERLAY");
	frameSlider.title:SetFontObject("GameFontHighLight");
	frameSlider.title:SetPoint("CENTER", frameSlider, "CENTER", 0, 30);
	frameSlider.title:SetText(title);
end

--Function for calculating the movement of a slider.
local function sliderCalc(frame, value)
	local halfStep = frame.stepValue / 2;
	value = value + halfStep - (value + halfStep) % frame.stepValue;
	_G[frame:GetName().."Text"]:SetText(value);
	return value;
end

--Function for selecting what mode history is stored, is it temporary or stored between sessions.
local function rollHistoryMode(bool)
	if bool then 
		variables.rollHistory = rollHistoryDB.rollHistory;
	else
		table.wipe(variables.rollHistory);
		table.wipe(rollHistoryDB.rollHistory);
	end
end

--Updating the config windows values.
local function updateConfigWindow()
	if not variables.addonLoaded then return end;
	
	llrConfigWindow.mainSettingFrame.editBox:SetNumber(llrSettings.mainRoll);
	llrConfigWindow.offSettingFrame.editBox:SetNumber(llrSettings.offRoll);
	llrConfigWindow.mogSettingFrame.editBox:SetNumber(llrSettings.mogRoll);
	llrConfigWindow.historySettingFrame.editBox:SetNumber(llrSettings.historyLength);
	llrConfigWindow.userScaleSlider:SetValue(llrSettings.userScale);
	llrConfigWindow.leaderScaleSlider:SetValue(llrSettings.leaderScale);
	
	_G["llrLeaderMode"]:SetChecked(llrSettings.leadMode);
	_G["llrScrollMode"]:SetChecked(llrSettings.scrollMode);
	_G["llrSaveMode"]:SetChecked(llrSettings.saveMode);
	_G["llrEventMode"]:SetChecked(llrSettings.eventMode);
	_G["llrAssistMode"]:SetChecked(llrSettings.assistWarning);
end

--Function for saving data from config window and closing it.
local function saveData()
	if not variables.addonLoaded then return end;
	
	local leadMode   	= _G["llrLeaderMode"]:GetChecked();
	local scrollMode   	= _G["llrScrollMode"]:GetChecked();
	local saveMode   	= _G["llrSaveMode"]:GetChecked();
	local eventMode     = _G["llrEventMode"]:GetChecked();
	local assistWarning = _G["llrAssistMode"]:GetChecked();
	
	local mainRoll   	= llrConfigWindow.mainSettingFrame.editBox:GetNumber();
	local offRoll   	= llrConfigWindow.offSettingFrame.editBox:GetNumber();
	local mogRoll 	  	= llrConfigWindow.mogSettingFrame.editBox:GetNumber();
	local historyLength = llrConfigWindow.historySettingFrame.editBox:GetNumber();
	
	if
		mainRoll 	  == offRoll or 
		mainRoll 	  == mogRoll or 
		offRoll  	  == mogRoll or
		mainRoll 	  == 0 or
		offRoll  	  == 0 or
		mogRoll 	  == 0 or
		historyLength == 0
	then
		print("");
		functions.printOut(
			"LLR: Invalid settings, please try again. Values cannot be the same or set to 0."
		);
		print("");
		return;
	end
	
	llrSettings.leadMode 	  = leadMode;
	llrSettings.scrollMode 	  = scrollMode;
	llrSettings.saveMode 	  = saveMode;
	llrSettings.eventMode	  = eventMode;
	llrSettings.assistWarning = assistWarning;
	
	llrSettings.mainRoll 	  = mainRoll;
	llrSettings.offRoll 	  = offRoll;
	llrSettings.mogRoll 	  = mogRoll;
	llrSettings.historyLength = historyLength;
	
	functions.controllEventListener();
	rollHistoryMode(saveMode);
	functions.historyLengthControl();
end

--Function for sharing user's roll settings.
local function shareSettings()
	C_ChatInfo.SendAddonMessage("lptlootroll", llrSettings.mainRoll .. "-" .. llrSettings.offRoll .. "-" .. llrSettings.mogRoll, "RAID");
end

local function setValuesToDefault()
	llrConfigWindow.mainSettingFrame.editBox:SetNumber(100);
	llrConfigWindow.offSettingFrame.editBox:SetNumber(50);
	llrConfigWindow.mogSettingFrame.editBox:SetNumber(6);
	llrConfigWindow.historySettingFrame.editBox:SetNumber(20);
	llrConfigWindow.userScaleSlider:SetValue(1);
	llrConfigWindow.leaderScaleSlider:SetValue(1);
	
	_G["llrLeaderMode"]:SetChecked(false);
	_G["llrScrollMode"]:SetChecked(false);
	_G["llrSaveMode"]:SetChecked(false);
	_G["llrEventMode"]:SetChecked(false);
end


-----------------------
-----EVENT HANDLING----
-----------------------


loadFrame:RegisterEvent("ADDON_LOADED");
loadFrame:SetScript("OnEvent", function(self, event, ...)
	local arg1,arg2,arg3,arg4 = ...;
	if event == "ADDON_LOADED" and arg1 == "LPT Loot Roll" then		
		userScale = llrSettings.userScale;
		leaderScale = llrSettings.leaderScale;
		updateConfigWindow();
		loadFrame:UnregisterEvent("ADDON_LOADED");
	elseif event == "CHAT_MSG_ADDON" and arg1 == "lptlootroll" then
		--Get the name of the sender, then display a pop up with said name and give it the data received.
		local name = strsplit("-", arg4);
		--print(name .. " " .. UnitName("player"));
		
		if name ~= UnitName("player") then 
			local dialog = StaticPopup_Show("llrSettingShare", name);
			if dialog then
				dialog.data = arg2;
			end
		end
	end
end);


-----------------------
-----CONFIG WINDOW-----
-----------------------


--Add config to standard wow interface window.
llrConfigWindow.name = "LPT Loot Roll";
InterfaceOptions_AddCategory(llrConfigWindow);


--Add logo image.
local t = llrConfigWindow:CreateTexture(nil,"BACKGROUND",nil,-8)
t:SetTexture("Interface\\AddOns\\LPT Loot Roll\\textures\\logo.tga")
t:SetSize(256,256);
t:SetPoint("CENTER", 0, 0);

--Create and set title of the config window.
llrConfigWindow.title = llrConfigWindow:CreateFontString(nil,"OVERLAY");
llrConfigWindow.title:SetFontObject("GameFontNormalLarge");
llrConfigWindow.title:SetPoint("CENTER", llrConfigWindow, "CENTER", 0, -140);
llrConfigWindow.title:SetText("Version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));

--Create checkboxes.
createCheckbutton("llrLeaderMode", llrConfigWindow, 12, -30, "Toggle leader mode", "If checked, the leader window will always open for each item.");
createCheckbutton("llrScrollMode", llrConfigWindow, 12, -60, "Reverse list direction", "If checked, lists will have highest values at top and lowest at bottom.");
createCheckbutton("llrSaveMode", llrConfigWindow, 12, -90, "Enable session memory.", "If checked, roll history will be kept between sessions.");
createCheckbutton("llrEventMode", llrConfigWindow, 12, -120, "Enable constant listener.", "If checked, the addon will ALWAYS listen for rolls even when the leader window is hidden.");
createCheckbutton("llrAssistMode", llrConfigWindow, 12, -150, "Enable assist loot.", "If checked, the addon will allow loot from all raid warnings.");

--Create frames to hold the roll settings.
llrConfigWindow.mainSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.offSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.mogSettingFrame 	= CreateFrame("Frame", nil, llrConfigWindow);
llrConfigWindow.historySettingFrame = CreateFrame("Frame", nil, llrConfigWindow);

--Create the editBox frames.
llrConfigWindow.mainSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.mainSettingFrame, "InputBoxTemplate");
llrConfigWindow.offSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.offSettingFrame, "InputBoxTemplate");
llrConfigWindow.mogSettingFrame.editBox 	= CreateFrame("EditBox", nil, llrConfigWindow.mogSettingFrame, "InputBoxTemplate");
llrConfigWindow.historySettingFrame.editBox = CreateFrame("EditBox", nil, llrConfigWindow.historySettingFrame, "InputBoxTemplate");

--Initialize the roll setting frames.
editBoxFactory(llrConfigWindow.mainSettingFrame, llrConfigWindow.mainSettingFrame.editBox, 12, 70, "Main Roll Max Value:", llrConfigWindow.offSettingFrame);
editBoxFactory(llrConfigWindow.offSettingFrame, llrConfigWindow.offSettingFrame.editBox, 12, 30, "Off Roll Max Value:", llrConfigWindow.mogSettingFrame);
editBoxFactory(llrConfigWindow.mogSettingFrame, llrConfigWindow.mogSettingFrame.editBox, 12, -10, "Mog Roll Max Value:", llrConfigWindow.historySettingFrame);

--Initialize the history length setting editbox.
editBoxFactory(llrConfigWindow.historySettingFrame, llrConfigWindow.historySettingFrame.editBox, 12, -50, "Max length of history:", llrConfigWindow.mainSettingFrame);

--Create the sliders.
llrConfigWindow.userScaleSlider   = CreateFrame("Slider", "LLR_User_Scaler", llrConfigWindow, "OptionsSliderTemplate");
llrConfigWindow.leaderScaleSlider = CreateFrame("Slider", "LLR_Leader_Scaler", llrConfigWindow, "OptionsSliderTemplate");

--Use factories to create the sliders, however add script outside of it as to prevent errors involving llrSettings global variable.
sliderFactory(llrConfigWindow, llrConfigWindow.userScaleSlider, -12, 100, "User window scale");
llrConfigWindow.userScaleSlider:SetScript("OnValueChanged", 
function (self, value)
	llrSettings.userScale = sliderCalc(llrConfigWindow.userScaleSlider, value);
	functions.updateUserScale();
end);

sliderFactory(llrConfigWindow, llrConfigWindow.leaderScaleSlider, -12, 30, "Leader window scale");
llrConfigWindow.leaderScaleSlider:SetScript("OnValueChanged", 
function (self, value)
	llrSettings.leaderScale = sliderCalc(llrConfigWindow.leaderScaleSlider, value);
	functions.updateLeaderScale();
end);

--Set valueis for the config window.
llrConfigWindow:SetScript("OnShow", function(self) loadFrame:RegisterEvent("CHAT_MSG_ADDON") updateConfigWindow() end);
llrConfigWindow:SetScript("OnHide", function(self) loadFrame:UnregisterEvent("CHAT_MSG_ADDON") end);
llrConfigWindow.default = function(self) setValuesToDefault() end;
llrConfigWindow.okay = function(self) saveData() end;
llrConfigWindow:Hide();

--Edit history editbox to have a max of two digits.
llrConfigWindow.historySettingFrame.editBox:SetMaxLetters(2);

--Create the share button.
llrConfigWindow.shareButton = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.shareButton, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -30, "Share", 120, 30, function(self) shareSettings() end);

--Create the clear data button.
llrConfigWindow.clearData = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.clearData, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -70, "Clear history", 120, 30, function(self) StaticPopup_Show("llrClearHistory") end);

--Create the reset position button.
llrConfigWindow.resetPosition = CreateFrame("Button", nil, llrConfigWindow, "GameMenuButtonTemplate");
functions.startButton(llrConfigWindow.resetPosition, llrConfigWindow, "TOPRIGHT", "TOPRIGHT", -12, -110, "Reset positions", 120, 30, function(self) StaticPopup_Show("llrResetPosition") end);



-----------------------
---GLOBAL FUNCTIONS----
-----------------------


--Create a "global" config toggle function so the settings window can be opened from the main lua file code.
function functions.toggleLlrConfig()
	InterfaceOptionsFrame_OpenToCategory(llrConfigWindow.name);
	if not llrConfigWindow:IsShown() then
		InterfaceOptionsFrame_OpenToCategory(llrConfigWindow.name);
	end
end


-----------------------
-----POPUP WINDOWS-----
-----------------------


--Pop up for accepting settings rolls.
StaticPopupDialogs["llrSettingShare"] = {
	text = "Use settings from %s?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self, message)
		--If we accept the data, parse it to numbers and store it in settings.
		local main, off, mog = strsplit("-", message);
		main = tonumber(main);
		off  = tonumber(off);
		mog  = tonumber(mog);
		if main and off and mog then
			llrConfigWindow.mainSettingFrame.editBox:SetNumber(main);
			llrConfigWindow.offSettingFrame.editBox:SetNumber(off);
			llrConfigWindow.mogSettingFrame.editBox:SetNumber(mog);
		end
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};