-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...;
local functions = {};
local variables = {};
local loadFrame = CreateFrame("FRAME");
local llr_EventFrame = CreateFrame("Frame");

local classArray = 
{
	DEATHKNIGHT = {"Plate","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces", "Two-Handed Maces","Polearms"},
	
	DEMONHUNTER = {"Leather","Warglaives","Fist Weapons", "One-Handed Axes", "One-Handed Swords"},
	
	DRUID = 	  {"Leather","One-Handed Maces","Two-Handed Maces","Polearms","Staves","Daggers","Fist Weapons"},
	
	HUNTER = 	  {"Mail","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","Polearms","Staves","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
	
	MAGE = 		  {"Cloth","One-Handed Swords","Staves","Daggers","Wands"},
	
	MONK = 		  {"Leather","One-Handed Axes","One-Handed Swords","One-Handed Maces","Polearms","Staves","Fist Weapons"},
	
	PALADIN = 	  {"Plate","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces","Two-Handed Maces", "Polearms"},
	
	PRIEST = 	  {"Cloth","One-Handed Maces","Staves","Daggers","Wands"},
	
	ROGUE = 	  {"Leather","One-Handed Axes","One-Handed Swords","One-Handed Maces","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
	
	SHAMAN = 	  {"Mail","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Maces","Two-Handed Maces","Staves","Daggers","Fist Weapons"},
	
	WARLOCK = 	  {"Cloth","One-Handed Swords","Staves","Daggers","Wands"},
	
	WARRIOR = 	  {"Plate","Shield","One-Handed Axes","Two-Handed Axes","One-Handed Swords","Two-Handed Swords","One-Handed Maces","Two-Handed Maces","Polearms","Staves","Daggers","Fist Weapons","Bows","Crossbows","Guns"},
};

local commandArray = 
{
	{
		name 		= "leader",
		func 		= 
			function() 
				functions.displayLeader()
			end,
		description = "Displays the raid leader window."
	},
	{
		name 		= "user",
		func 		= 
			function() 
				functions.displayUser()
			end,
		description = "Displays the normal user window."
	},
	{
		name		= "clear",
		func 		= 
			function() 
				StaticPopup_Show("llrClearHistory")
			end,
		description = "Clear all stored roll data."
	},
	{
		name 		= "reset",
		func		= 
			function() 
				StaticPopup_Show("llrResetPosition")
			end,
		description = "Resets position of windows."
	},
	{
		name 		= "config",
		func		= 
			function() 
				functions.toggleLlrConfig()
			end,
		description = "Open the config menu"
	}
};

--Create a sub-namespace for the global function.
namespace.functions = functions;
namespace.variables = variables;
variables.rollHistory = {};


-----------------------
--FUNCTION DEFINITIONS-
-----------------------


--If the scale is larger than 1200 then take parentScale into consideration, otherwise just divide with scale. 
local function calculateScale()
	local newScale, yResolution, parentScale = 1, string.match( GetCVar( "gxWindowedResolution" ), "%d+x(%d+)" ), UIParent:GetScale();
	if tonumber(yResolution) > 1200 then
		newScale = 768/yResolution/parentScale;
	else
		newScale = 768/yResolution;
	end
	return newScale;
end

--Function for handling manual rolls.
local function manualRoll(self)
	local number = tonumber(self.editBox:GetText());
	if number ~= nil then
		if functions.isUserShown() then
			functions.hideUser();
		end
		
		if functions.isLeaderShown() then
			functions.disableLeader();
		end
		
		RandomRoll(1,number);
	end
	self:Hide()
end

--Slash commands.
SLASH_LLR1 = "/llr";
SLASH_LLR2 = "/lptlootroll";
SLASH_LLR3 = "/lpt";

SlashCmdList.LLR = function(msg)
	msg = string.lower(msg);
	
	for i = 1, #commandArray do
		if string.find(msg or "", commandArray[i].name) then
			commandArray[i].func();
			return;
		end
	end
	
	print("");
	functions.printOut(GetAddOnMetadata("LPT Loot Roll", "Title") .. " version: " .. GetAddOnMetadata("LPT Loot Roll", "Version"));
	
	if(msg == nil or msg == "") then  
		functions.printOut("Error: No command recieved.");
	else
		functions.printOut("Error: Command not recognized.");
	end
	
	print("");
	functions.printOut("Available commands are:");
	
	for i = 1, #commandArray do
		functions.printOut(commandArray[i].name:gsub("^%l", string.upper) .. " - " .. commandArray[i].description);
	end
	
	print("");
end


-----------------------
-----EVENT HANDLING----
-----------------------


--Initial check to see if addon variables exist, if they don't, create them.
loadFrame:RegisterEvent("ADDON_LOADED");
loadFrame:SetScript("OnEvent", function(self, event, addonName, ...)
	if event == "ADDON_LOADED" and addonName == "LPT Loot Roll" then
		if llrSettings == nil then 
			llrSettings = {};
		end
		
		--UI positions
		
		if llrSettings.raidLeadWindowPos == nil then
			llrSettings.raidLeadWindowPos = {["point"] = "CENTER", ["relativePoint"] = "CENTER", ["xOfs"] = 0, ["yOfs"] = 0};
		end
		
		if llrSettings.userWindowPos == nil then
			llrSettings.userWindowPos = {["point"] = "CENTER", ["relativePoint"] = "CENTER", ["xOfs"] = 0, ["yOfs"] = 0};
		end
		
		--Booleans
		
		if llrSettings.leadMode == nil then
			llrSettings.leadMode = false;
		end
		
		if llrSettings.scrollMode == nil then
			llrSettings.scrollMode = false;
		end
		
		if llrSettings.saveMode == nil then
			llrSettings.saveMode = false;
		end
		
		if llrSettings.eventMode == nil then
			llrSettings.eventMode = false;
		end
		
		if llrSettings.assistWarning == nil then
			llrSettings.assistWarnings = false;
		end
		
		--Integers
		
		if llrSettings.mainRoll == nil then
			llrSettings.mainRoll = 100;
		end
		
		if llrSettings.offRoll == nil then
			llrSettings.offRoll = 50;
		end
		
		if llrSettings.mogRoll == nil then
			llrSettings.mogRoll = 25;
		end
		
		if llrSettings.historyLength == nil then
			llrSettings.historyLength = 20;
		end
		
		if llrSettings.leaderScale == nil then
			llrSettings.leaderScale = 1;
		end
		
		if llrSettings.userScale == nil then
			llrSettings.userScale = 1;
		end
		
		--History database control.
		
		if rollHistoryDB == nil then 
			rollHistoryDB = {};
		end
		
		if rollHistoryDB.rollHistory == nil then
			rollHistoryDB.rollHistory = {};
		end
		
		--Save mode binding.
		
		if llrSettings.saveMode then
			variables.rollHistory = rollHistoryDB.rollHistory;
			
			--Pre cache history items.
			for i, v in ipairs(variables.rollHistory) do
				GetItemInfo(v.item);
			end
		end
		
		variables.screenScale = calculateScale();
		variables.addonLoaded = true;
		loadFrame:UnregisterEvent("ADDON_LOADED");
	end
end);

llr_EventFrame:RegisterEvent("CHAT_MSG_RAID_WARNING");
llr_EventFrame:SetScript("OnEvent", function(self,event,...)
	if(event == "CHAT_MSG_RAID_WARNING") then
		local arg1, arg2 = ...;
		local name, _ = strsplit("-", arg2);
		
		--Check if the person sending a raidwarning is the raid leader.
		if UnitIsGroupLeader(name) or llrSettings.assistWarning then
			local _,playerClass = UnitClass("Player");
			local _,link,_,_,_,itemType,itemSubType,_,invType = GetItemInfo(arg1);
			if not (link == nil or link == "") and (itemType == "Armor" or itemType == "Weapon" or itemSubType == "Companion Pets") then
				--Determine if the player's class can equip the item.
				local boolean = false;
				
				for i,v in ipairs(classArray[playerClass]) do
					if(v==itemSubType) then
						boolean = true;
						break;
					end
				end
				
				table.insert(variables.rollHistory, 1, 
				{
					item 		= link,
					mainList 	= {},
					offList 	= {},
					mogList 	= {},
					otherList   = {}
				});
				
				functions.historyLengthControl();
				
				--Determine the equipable boolean. Check if it's an equipable type or if it's subtype is misc (neck, trinket, ring) or if the item is a cloak.
				local isEquipable = boolean or (itemSubType == "Miscellaneous" or invType == "INVTYPE_CLOAK" or itemSubType == "Companion Pets");
			
				if UnitIsGroupLeader("Player") or llrSettings.leadMode then
					functions.leaderEvent(isEquipable);
				else
					functions.userEvent(isEquipable);
				end
			end 
		end
	end 
end);


-----------------------
---GLOBAL FUNCTIONS----
-----------------------


--Custom printout method for yellow message printouts.
function functions.printOut(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,1,1,0);
end

--Input the current item into the roll frame.
function functions.setItemButtonData(frame, link)
	frame:SetScript("OnMouseDown", 
	function(self,button) 
		if(IsShiftKeyDown() and button=="RightButton") then 
			OpenAzeriteEmpoweredItemUIFromLink(link) 
		elseif (IsControlKeyDown() and button=="LeftButton") then 
			DressUpItemLink(link) 
		end 
	end);
	
	frame:SetNormalTexture(select(10,GetItemInfo(link)));
	
	frame:SetScript("OnEnter", 
	function(self) 
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR"); 
		GameTooltip:ClearLines(); 
		GameTooltip:SetHyperlink(link); 
		GameTooltip:Show();
	end);
end

--Button factory to be used by both the leader and user window.
function functions.startButton(button, frame, firstAnchor, secondAnchor, xValue, yValue, title, xLength, yLength, func)
	button:SetPoint(firstAnchor, frame, secondAnchor, xValue, yValue);
	button:SetSize(xLength,yLength);
	button:SetText(title);
	button:SetScript("OnClick", func);
end

--Function for controlling length of history array.
function functions.historyLengthControl()
	while # variables.rollHistory > llrSettings.historyLength do
		table.remove(variables.rollHistory);
	end
end


-----------------------
-----POPUP WINDOWS-----
-----------------------


--Pop up for manual rolls.
StaticPopupDialogs["llrManualRoll"] = 
{
	text = "Enter roll (Max: 9999):",
	button1 = "Accept",
	button2 = "Cancel",
	hasEditBox = 1,
	maxLetters = 4,
	OnShow = function(self)
		local editBox = self.editBox;
		editBox:SetText("");
		editBox:SetFocus();
	end,
	OnAccept = function(self)
		manualRoll(self);
	end,
	EditBoxOnEnterPressed = function(self)
		manualRoll(self:GetParent());
	end,
	EditBoxOnEscapePressed = function(self)
		self:GetParent():Hide();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting history.
StaticPopupDialogs["llrClearHistory"] = 
{
	text = "Are you sure you wish to reset roll history?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self)
		functions.printOut("LLR: Item history has been reset.");
		functions.resetRolls();
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};

--Pop up for resetting locations.
StaticPopupDialogs["llrResetPosition"] = 
{
	text = "Are you sure you wish to reset window positions?",
	button1 = "Accept",
	button2 = "Cancel",
	OnAccept = function(self)
		functions.printOut("LLR: Location of windows have been reset.");
		functions.resetUserWindow();
		functions.resetLeaderWindow();		
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1
};