-----------------------
----LOCAL VARIABLES----
-----------------------


local ADDON_NAME, namespace = ...
local functions = namespace.functions;
local variables = namespace.variables;
local userWindow = CreateFrame("Frame", "LPTLootRollUser", nil, "BasicFrameTemplateWithInset");
local loadFrame = CreateFrame("FRAME");


-----------------------
-----EVENT HANDLING----
-----------------------


loadFrame:RegisterEvent("ADDON_LOADED"); 
loadFrame:RegisterEvent("PLAYER_LOGOUT"); 

loadFrame:SetScript("OnEvent", function(self, event, addonName)
	if event == "ADDON_LOADED" and addonName == "LPT Loot Roll" then
		userWindow:SetPoint(llrSettings.userWindowPos.point, nil, llrSettings.userWindowPos.relativePoint, llrSettings.userWindowPos.xOfs, llrSettings.userWindowPos.yOfs);
		loadFrame:UnregisterEvent("ADDON_LOADED");
	elseif event == "PLAYER_LOGOUT" then
		llrSettings.userWindowPos.point, _, llrSettings.userWindowPos.relativePoint, llrSettings.userWindowPos.xOfs, llrSettings.userWindowPos.yOfs = userWindow:GetPoint();
	end
end);


-----------------------
-------USER WINDOW-----
-----------------------


userWindow:SetSize(200,100);
userWindow:SetMovable(true);
userWindow:EnableMouse(true);
userWindow:RegisterForDrag("LeftButton");
userWindow:SetScript("OnDragStart", userWindow.StartMoving)
userWindow:SetScript("OnDragStop", userWindow.StopMovingOrSizing)
userWindow:SetScript("OnShow", function(self) 
	functions.updateUserScale();
end);
userWindow.title = userWindow:CreateFontString(nil,"OVERLAY");
userWindow.title:SetFontObject("GameFontHighLight");
userWindow.title:SetPoint("LEFT", userWindow.TitleBg, "left", 5, -1);
userWindow.title:SetText("LLR User");
userWindow:SetClampedToScreen(true);
userWindow:SetClampRectInsets(0, 0, 0, 0);
userWindow:Hide();

--USER WINDOW ITEM PREVIEW
userWindow.itemButton = CreateFrame("Button", nil, userWindow, "GameMenuButtonTemplate");
userWindow.itemButton:SetPoint("LEFT", userWindow, "LEFT", 10, -10);
userWindow.itemButton:SetSize(60,60);
userWindow.itemButton:SetScript("OnLeave", function(self) GameTooltip:Hide() ResetCursor() end);

userWindow.itemButton.itemTexture = userWindow.itemButton:CreateTexture(nil,"OVERLAY");
userWindow.itemButton.itemTexture:SetPoint("TOPLEFT", 0, 0);
userWindow.itemButton.itemTexture:SetPoint("BOTTOMRIGHT", 0, 0);

--USER WINDOW BUTTONS
userWindow.mainSpeccButton = CreateFrame("Button", nil, userWindow, "GameMenuButtonTemplate");
userWindow.offSpeccButton = CreateFrame("Button", nil, userWindow, "GameMenuButtonTemplate");
userWindow.mogButton = CreateFrame("Button", nil, userWindow, "GameMenuButtonTemplate");
userWindow.otherButton = CreateFrame("Button", nil, userWindow, "GameMenuButtonTemplate");

--START INSTANCE OF USER WINDOW BUTTONS
functions.startButton(userWindow.mainSpeccButton, userWindow, "CENTER", "TOP", 0, -45, "Main", 60, 30, function(self) RandomRoll(1,llrSettings.mainRoll) userWindow:Hide() end);
functions.startButton(userWindow.offSpeccButton, userWindow, "CENTER", "TOP", 0, -75, "Off", 60, 30, function(self) RandomRoll(1,llrSettings.offRoll) userWindow:Hide() end);
functions.startButton(userWindow.mogButton, userWindow, "CENTER", "TOP", 60, -45, "Mog", 60, 30, function(self) RandomRoll(1,llrSettings.mogRoll) userWindow:Hide() end);
functions.startButton(userWindow.otherButton, userWindow, "CENTER", "TOP", 60, -75, "Other", 60, 30, function(self) StaticPopup_Show("llrManualRoll") end);


-----------------------
---GLOBAL FUNCTIONS----
-----------------------

--Function for updating the user window scale.
function functions.updateUserScale()
	userWindow:SetScale(variables.screenScale*llrSettings.userScale);
end

--Function for starting the user window upon new item drop.
function functions.userEvent(isEquipable)
	if isEquipable then
		--Setup the item "frame" with tooltip, mog and artifact system implementation.
		functions.setItemButtonData(userWindow.itemButton, variables.rollHistory[1].item);
		userWindow:Show();
	end
end

--Function for displaying the user window.
function functions.displayUser()
	if not userWindow:IsShown() then
		userWindow:Show();
	end 
end

--Function for hiding the user window.
function functions.hideUser()
	if userWindow:IsShown() then
		userWindow:Hide();
	end 
end

--Function to determine if the user window is displayed.
function functions.isUserShown()
	return userWindow:IsShown();
end

--Function for resetting the user window position.
function functions.resetUserWindow()
	userWindow:ClearAllPoints();
	userWindow:SetPoint("CENTER", UIParent, "CENTER");
end